<?php
/**
 * 2007-2020 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2020 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License
 *     (AFL 3.0) International Registered Trademark & Property of PrestaShop SA
 */

if ( ! defined('_PS_VERSION_')) {
    exit;
}

include_once __DIR__.'/base.php';

class Multiparcelsshipping extends Multiparcelsshipping_Base
{

    const NAME = 'multiparcelsshipping';

    const VERSION = '0.0.6';

    const TABLE_LOCATIONS = 'multiparcelsshipping_locations';

    const TABLE_CART = 'cart_multiparcelsshipping';

    const TABLE_CARRIER = 'carrier_multiparcelsshipping';

    const ACTION_RELOAD = 'reload';

    const ACTION_ENABLE_CARRIER = 'enable_carrier';

    const ACTION_DISABLE_CARRIER = 'disable_carrier';

    const ACTION_SHOW_LOCATIONS = 'show_locations';

    public function __construct()
    {
        $this->name          = 'multiparcelsshipping';
        $this->tab           = 'shipping_logistics';
        $this->version       = self::VERSION;
        $this->author        = 'MultiParcels';
        $this->author_uri    = 'https://multiparcels.com';
        $this->need_instance = 1;
        $this->bootstrap     = true;

        parent::__construct();

        $this->displayName = $this->l('MultiParcels Shipping & Labels');
        $this->description = $this->l('Easiest, fastest and the cheapest way to integrate carriers with all deliveries methods to send parcels with just a few button clicks.');

        $this->ps_versions_compliancy = ['min' => '1.7', 'max' => _PS_VERSION_];

        $carrierTranslations = [
            'courier'   => $this->l('courier'),
            'terminals' => $this->l('terminals'),
        ];

        $this->configuration = new MultiParcels_Shipping_Configuration($this);
        $this->api           = new MultiParcels_Shipping_Api($this);
        $this->apiClient     = new MultiParcels_Shipping_Api_Client($this);
        $this->carriers      = new MultiParcels_Shipping_Carriers($this,
            $carrierTranslations);
        $this->permissions   = new MultiParcels_Shipping_Permissions($this);
        $this->locations     = new MultiParcels_Shipping_Locations($this);
        $this->hooks         = new MultiParcels_Shipping_Hooks($this);
        $this->helper        = new MultiParcels_Shipping_Helper($this);
    }

    public function install()
    {
        if (extension_loaded('curl') == false) {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module');

            return false;
        }

        $setup = new MultiParcels_Shipping_Setup($this);

        if ( ! parent::install()) {
            return false;
        }

        return $setup->install();
    }

    public function uninstall()
    {
        $setup = new MultiParcels_Shipping_Setup($this);
        $setup->uninstall();

        return parent::uninstall();
    }

    public function getContent()
    {
        $action           = Tools::getValue('action');
        $updateSuccessful = Tools::getValue('update_successful');

        if ($action == self::ACTION_RELOAD) {
            $count = $this->api->reload();

            $this->helper->redirectAfterReload($count);
        }

        if ($action == self::ACTION_ENABLE_CARRIER) {
            $this->carriers->enable(Tools::getValue('carrier_code'));

            $this->helper->redirectToConfiguration();
        }

        if ($action == self::ACTION_DISABLE_CARRIER) {
            $this->carriers->disable(Tools::getValue('carrier_code'));

            $this->helper->redirectToConfiguration();
        }

        if (((bool)Tools::isSubmit('submitMultiparcelsshippingModuleFreeApi')) == true) {
            $this->api->requestApiKey(Tools::getValue('multiparcels_free_api_email'));
            $this->helper->redirectToReload();
        }

        if (((bool)Tools::isSubmit('submitMultiparcelsshippingModule')) == true) {
            $this->postProcess();
        }

        $permissionTranslations = [
            MultiParcels_Shipping_Permissions::CODE_NONE    => $this->l('None. Wrong API key.'),
            MultiParcels_Shipping_Permissions::CODE_LIMITED => $this->l('Limited'),
            MultiParcels_Shipping_Permissions::CODE_FULL    => $this->l('Full'),
        ];

        $variables = [
            'reload_link'         => $this->context->link->getAdminLink('AdminModules',
                true,
                [],
                [
                    'configure' => 'multiparcelsshipping',
                    'action'    => self::ACTION_RELOAD,
                ]),
            'configure_link'      => $this->context->link->getAdminLink('AdminModules',
                true,
                [],
                [
                    'configure' => 'multiparcelsshipping',
                ]),
            'carriers_link'       => $this->context->link->getAdminLink('AdminCarriers',
                true,
                [],
                []),
            'show_locations_link' => $this->context->link->getAdminLink('AdminModules',
                true,
                [],
                [
                    'configure' => 'multiparcelsshipping',
                    'action'    => self::ACTION_SHOW_LOCATIONS,
                ]),
            'api_key'             => $this->configuration->getApiKey(),
            'module_dir'          => $this->_path,
            'admin_email'         => $this->context->employee->email,
            'last_update_at'      => $this->configuration->getLastUpdateAt(),
            'carriers'            => $this->configuration->getCarriers(),
            'update_successful'   => $updateSuccessful,
            'permissions'         => $this->permissions->get(),
            'permissions_text'    => $permissionTranslations[$this->permissions->get()],
            'show_locations'      => false,
            'cron_url'            => Tools::getHttpHost(true).__PS_BASE_URI__.sprintf('index.php?fc=module&module=%s&controller=cron&token=%s',
                    self::NAME, $this->configuration->getCronToken()),
            'version_check'       => $this->api->checkLatestVersion(),
        ];

        if ($action == self::ACTION_SHOW_LOCATIONS) {
            $variables['show_locations'] = true;
            $variables['locations']      = $this->locations->all();
        }

        if (isset($variables['carriers']['carriers'])) {
            foreach ($variables['carriers']['carriers'] as $key => $value) {
                $variables['carriers']['carriers'][$key]['has_locations'] = true;

                $variables['carriers']['carriers'][$key]['enabled'] = $this->configuration->checkCarrier($value['carrier_code']);

                if ($value['has_pickup_points'] || $value['has_terminals']) {
                    $variables['carriers']['carriers'][$key]['has_locations'] = $this->locations->hasLocations($value['carrier_code']);
                }
                $variables['carriers']['carriers'][$key]['enable_link']  = $this->context->link->getAdminLink('AdminModules',
                    true,
                    [],
                    [
                        'configure'    => 'multiparcelsshipping',
                        'action'       => self::ACTION_ENABLE_CARRIER,
                        'carrier_code' => $value['carrier_code'],
                    ]);
                $variables['carriers']['carriers'][$key]['disable_link'] = $this->context->link->getAdminLink('AdminModules',
                    true,
                    [],
                    [
                        'configure'    => 'multiparcelsshipping',
                        'action'       => self::ACTION_DISABLE_CARRIER,
                        'carrier_code' => $value['carrier_code'],
                    ]);
            }
        }

        foreach ($variables as $key => $value) {
            $this->context->smarty->assign($key, $value);
        }

        $this->context->smarty->fetch($this->local_path.'views/templates/admin/configure.tpl');

        return $this->context->smarty->fetch($this->local_path.'views/templates/admin/configure.tpl');
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $oldApiKey = $this->configuration->getApiKey();
        $newApiKey = Tools::getValue('api_key');

        if ($oldApiKey != $newApiKey) {
            $this->configuration->setApiKey($newApiKey);

            $this->helper->redirectToReload();
        }
    }

    public function hookBackOfficeHeader()
    {
        $this->context->controller->addCSS($this->_path.'views/css/backoffice.css');
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path.'/views/js/select2.full.min.js');
        $this->context->controller->addJS($this->_path.'/views/js/front.js');
        $this->context->controller->addCSS($this->_path.'/views/css/front.css');
        $this->context->controller->addCSS($this->_path.'/views/css/select2.min.css');

        Media::addJsDef([
            "multiparcels" => [
                'translations' => [
                    'working_hours'             => $this->l('Working hours'),
                    'pickup_location_not_found' => $this->l('Pickup location not found'),
                ],
            ],
        ]);
    }

    public function hookActionValidateStepComplete($params)
    {
        $text = $this->l('Please select the pickup location');
        $this->hooks->checkIfPickupIsSelectedInCheckout($params, $text);
    }

}
